package AST;


public interface ASTVisitor {

	void visitReturnStatement(ASTReturnStatement ret);
	void visitPrintStatement(ASTPrintStatement ps);
	void visitIfStatement(ASTIfStatement stat);
	void visitWhileStatement(ASTWhileStatement stat);
	void visitAddExpression(ASTAddExpression expr);
	void visitCompareExpression(ASTCompareExpression expr);
	void visitCompoundStatement(ASTCompoundStatement compound);
	void visitExpressionStatement(ASTExpressionStatement expressionStatement);
	void visitAssignExpression(ASTAssignExpression assignExpression);
	void visitMulExpression(ASTMulExpression mulExpression);
	void visitReferenceExpression(ASTReferenceExpression ref);
	void visitConstExpression(ASTConstExpression conste);
	
	/**
	 * contains default implementations of all visitor functions so
	 * that the all subexpressions of an expression are visited.
	 * (but we don't traverse compoundstatement or into if/while bodies).
	 */
	public static abstract class DefaultExpressionVisitor implements ASTVisitor {
		@Override
		public void visitAddExpression(ASTAddExpression expr) {
			expr.getLeft().accept(this);
			expr.getRight().accept(this);
		}

		@Override
		public void visitAssignExpression(ASTAssignExpression assignExpression) {
			assignExpression.getExpression().accept(this);
		}

		@Override
		public void visitCompareExpression(ASTCompareExpression expr) {
			expr.getLeft().accept(this);
			expr.getRight().accept(this);
		}

		@Override
		public void visitCompoundStatement(ASTCompoundStatement compound) {
		}

		@Override
		public void visitConstExpression(ASTConstExpression conste) {
		}

		@Override
		public void visitExpressionStatement(
				ASTExpressionStatement expressionStatement) {
			expressionStatement.getExpression().accept(this);
		}

		@Override
		public void visitIfStatement(ASTIfStatement stat) {
			stat.getCondition().accept(this);
		}

		@Override
		public void visitMulExpression(ASTMulExpression mulExpression) {
			mulExpression.getLeft().accept(this);
			mulExpression.getRight().accept(this);
		}

		@Override
		public void visitPrintStatement(ASTPrintStatement ps) {
			ps.getExpression().accept(this);
		}

		@Override
		public void visitReferenceExpression(ASTReferenceExpression ref) {
		}

		@Override
		public void visitReturnStatement(ASTReturnStatement ret) {
			ret.getExpression().accept(this);
		}

		@Override
		public void visitWhileStatement(ASTWhileStatement stat) {
			stat.getCondition().accept(this);
		}
	}
	
	/**
	 * Base visitor class with all expression visitors initialized to an empty method.
	 * (Useful base when you only want to look at statements)
	 */
	public static abstract class StatementVisitor implements ASTVisitor {
		@Override
		public void visitAddExpression(ASTAddExpression expr) { }
		@Override
		public void visitAssignExpression(ASTAssignExpression assignExpression) { }
		@Override
		public void visitCompareExpression(ASTCompareExpression expr) {	}
		@Override
		public void visitConstExpression(ASTConstExpression conste) { }
		@Override
		public void visitMulExpression(ASTMulExpression mulExpression) { }
		@Override
		public void visitReferenceExpression(ASTReferenceExpression ref) { }		
	}
}
