package DataFlow;
import CFG.CFG;

/**
 * Abstraction for different dataflow analyses.
 * (This is a generic interface which can be parameterized by the
 *  datatype used to represent dataflow values).
 * @author Matthias Braun
 */
public interface DataFlowInterface<Data, CFGNode> {
	/**
	 * Prepare dataflow analysis to work on a CFG.
	 */
	void initialize(CFG<CFGNode> cfg);
	
	/**
	 * return true if analysis is a backwards analysis.
	 * false if it is a forward analysis
	 */
	boolean isBackward();
	
	/**
	 * join/union operation for 2 data values
	 * The Data returned must not be null
	 * (null is reserved by dataflow algorithm as marker for not calculated values)
	 */
	Data union(Data d1, Data d2);
	
	/**
	 * apply transfer function for a node in the CFG 
	 * The Data returned must not be null
	 * (null is reserved by dataflow algorithm as marker for not calculated values)
	 * Warning: It is not allowed to modify the object from the data Parameter!
	 *          You have to copy/clone it first.
	 */
	Data transfer(CFGNode node, Data data);	
}
